/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *         (C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include <pkgupgradecontroller.h>
#include <dbsdglobal.h>
#include <dialogs.h>
#include <packagesettings.h>

/* Qt */
#include <qframe.h>
#include <qprogressbar.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qpixmap.h>
#include <qpushbutton.h>

/* KDE/KParts */
#include <kparts/part.h>
#include <klibloader.h>
#include <kde_terminal_interface.h>
#include <kprocess.h>

#include "frmpkgupgradecontroller.h"

FrmPkgUpgradeController::FrmPkgUpgradeController(QWidget* parent, const char* name, WFlags fl)
	:	uiFrmPkgUpgradeController(parent, name,  fl),
		commandString(""),
		currentMessageType(None),
		finished(false),
		canceled(false)
{
	KLibFactory* factory = KLibLoader::self()->factory("libkonsolepart");
	if(factory == NULL) {
		Dialogs::warnDlg(tr("Error: No Konsole installed!"));;
		return;
	}

	terminalPart = static_cast<KParts::Part*>(factory->create(frTerminal, NULL, "QObject", "KParts::ReadOnlyPart"));
	terminalInterface = static_cast<ExtTerminalInterface*>(terminalPart->qt_cast("ExtTerminalInterface"));

	if(terminalInterface == NULL) {
		Dialogs::warnDlg(tr("Error: Can't access the extended Konsole interface.\n\n"
								  "Make sure you are using KDE 3.2 or later."));
		return;
	}

	terminalInterface->setAutoStartShell(false);
	terminalInterface->setAutoDestroy(false);

	frTerminalLayout->addWidget(terminalPart->widget(), 0, 0);

	controller = new PkgUpgradeController(this);
	connect(terminalPart, SIGNAL(receivedData(const QString &)), controller, SLOT(outputReceived(const QString &)));
	connect(terminalPart, SIGNAL(processExited(KProcess *)), SLOT(processExited(KProcess *)));
	connect(controller, SIGNAL(informationMessage(const QString &)), SLOT(informationMessage(const QString &)));
	connect(controller, SIGNAL(progressMessage(const QString &)), SLOT(progressMessage(const QString &)));
	connect(controller, SIGNAL(warningMessage(const QString &)), SLOT(warningMessage(const QString &)));
	connect(controller, SIGNAL(operationProgressChanged(int)), SLOT(operationProgressChanged(int)));
	connect(controller, SIGNAL(currentPackageChanged(const QString &)), lblCurrentPackage, SLOT(setText(const QString &)));
	connect(controller, SIGNAL(overallProgressChanged(int)), pbTotalProgress, SLOT(setProgress(int)));

	frTerminal->hide();

	DBSDIconLoader *il = DBSDGlobal::getIconLoader();
	btnShowHideTerminal->setIconSet(il->loadIcon("view_detailed", DBSDIconLoader::Button));
	btnCancelClose->setIconSet(il->loadIcon("button_cancel", DBSDIconLoader::Button));
	pmNone = il->getBlankPixmap(16, 16);
	pmProgress = il->loadIcon("ok", DBSDIconLoader::Size16x16);
	pmWarning = il->loadIcon("warning", DBSDIconLoader::Size16x16);
	pmInfo = il->loadIcon("info", DBSDIconLoader::Size16x16);
	pmClose = il->loadIcon("fileclose", DBSDIconLoader::Button);
	lblPreviousMessageIcon->setPixmap(pmNone);
	lblCurrentMessageIcon->setPixmap(il->loadIcon("1rightarrow", DBSDIconLoader::Size16x16));

	busyIndicatorTimer = new QTimer(this);
	connect(busyIndicatorTimer, SIGNAL(timeout()), SLOT(moveBusyIndicator()));
	enableBusyIndicatorTimer(true);

	connect(btnCancelClose, SIGNAL(clicked()), SLOT(close()));
}


void FrmPkgUpgradeController::closeEvent(QCloseEvent *e) {
	if(finished) {
		setResult(canceled ? Rejected : Accepted);
		e->accept();
	} else {
		e->ignore();

		canceled = Dialogs::question(tr("Canceling the installation or upgrade of packages is\n"
													"DANGEROUS and cause damage to installed software and\n"
													"the package database! NEVER do this when a package\n"
													"is currently being installed, uninstalled or registered!\n"
													"\n"
													"Do you really want to proceed?"));
	
		if(canceled)
			controller->killAll();
	}
}


void FrmPkgUpgradeController::processExited(KProcess *process) {
	finished = true;

	pbTotalProgress->setProgress(100);
	pbCurrentMessage->hide();
	enableBusyIndicatorTimer(false);

	btnCancelClose->setText(tr("Close"));
	btnCancelClose->setIconSet(pmClose);

	if(canceled) {
		message(Warning, tr("- Canceled. Click \"Details\" for more information."));

		Dialogs::infoDlg(tr("All operations were canceled.\n"
								"\n"
								"There are probably files left from an unfinished build.\n"
								"Please consider running the Cleaning Wizard to remove\n"
								"these files from your system."));
	} else {
		message(Information, tr("- All operations finished. Click \"Details\" for more information."));

		if(!process->normalExit() || process->exitStatus() != 0) {
			const QString failureMsg = tr("Installation/upgrade of some packages failed.\n"
													"\n"
													"Possible reasons:\n"
													"\n"
													"- Some of the installed packages need to be newer than\n"
													"  the currently installed ones. You should consider\n"
													"  upgrading these packages.\n"
													"%1\n"
													"\n"
													"You can scroll with the mouse wheel to find out what the\n"
													"error was. Click 'Close' when you're done.");
			
			const QString binReasons = tr("\n"
													"- Binary packages for the latest version of that software are not\n"
													"  available yet. Usually, it takes a few days to prepare them.\n"
													"\n"
													"- Redistribution of certain packages in binary form is restricted.\n"
													"  This applies especially to packages that use patented algorithms\n"
													"  (MP3 encoding, for example) and software that is only free for\n"
													"  \"non-commercial\" use. Other packages that require such packages\n"
													"  might also not be available in binary form.");
			
			Dialogs::warnDlg(failureMsg.arg((PackageSettings().getPackageUse() == 2) ? binReasons : ""));
		}
	}
}


void FrmPkgUpgradeController::moveBusyIndicator() {
	pbCurrentMessage->setProgress(pbCurrentMessage->progress() + 1);
}


void FrmPkgUpgradeController::enableBusyIndicatorTimer(bool enable) {
	if(enable)
		busyIndicatorTimer->start(23, false);
	else
		busyIndicatorTimer->stop();
}


void FrmPkgUpgradeController::progressMessage(const QString & m) {
	message(Progress, m);
}


void FrmPkgUpgradeController::warningMessage(const QString & m) {
	message(Warning, m);
}


void FrmPkgUpgradeController::informationMessage(const QString & m) {
	message(Information, m);
}


void FrmPkgUpgradeController::message(MessageType type, const QString & string) {
	lblSecondPreviousMessageIcon->setPixmap(*lblPreviousMessageIcon->pixmap());
	lblSecondPreviousMessage->setText(lblPreviousMessage->text());

	lblPreviousMessage->setText(lblCurrentMessage->text());
	if(currentMessageType == Information)
		lblPreviousMessageIcon->setPixmap(pmInfo);
	else if(currentMessageType == Warning)
		lblPreviousMessageIcon->setPixmap(pmWarning);
	else if(currentMessageType == Progress)
		lblPreviousMessageIcon->setPixmap(pmProgress);
	else
		lblPreviousMessageIcon->setPixmap(pmNone);

	currentMessageType = type;
	lblCurrentMessage->setText(string);

	pbCurrentMessage->setTotalSteps(0);
	pbCurrentMessage->setPercentageVisible(false);
	enableBusyIndicatorTimer(true);
}


void FrmPkgUpgradeController::operationProgressChanged(int percent) {
	pbCurrentMessage->show();
	enableBusyIndicatorTimer(false);
	pbCurrentMessage->setTotalSteps(100);
	pbCurrentMessage->setPercentageVisible(true);
	pbCurrentMessage->setProgress(percent);
}


void FrmPkgUpgradeController::setCommand(QStringList args) {
	QStrList slArgs;
	for (QStringList::Iterator it = args.begin(); it != args.end(); ++it)
      	slArgs.append((*it).latin1());

	setCommand(args.first(), slArgs);
}


void FrmPkgUpgradeController::setCommand(const QString & cmdString, QStrList args) {
	commandString = cmdString;
	commandArguments = args;
}


void FrmPkgUpgradeController::polish() {
	uiFrmPkgUpgradeController::polish();
	adjustSize();

	if(!commandString.isEmpty())
		terminalInterface->startProgram(commandString, commandArguments);
	else
		Dialogs::warnDlg("Error: No command specified!");
}
